/* -- $Id: Converter.java,v 1.9 2003/11/06 07:00:53 kshaikh Exp $ --
 Copyright (C) 2001 Auriga Logic <http://www.aurigalogic.com>
 Author: Khurshidali Shaikh <kshaikh@aurigalogic.com>

 ============================================================================
		    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
 ============================================================================

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
package com.aurigalogic.aurigadoc.cmdline;

import com.aurigalogic.aurigadoc.core.*;
import com.aurigalogic.aurigadoc.logger.*;
import com.aurigalogic.aurigadoc.util.JHelpViewer;

import java.util.Properties;

/**
 * This class provides a command line interface for the aurigadoc converter.
 *
 * @author <a href="mailto:kshaikh@aurigalogic.com">Khurshidali Shaikh</a>
 * @version $Revision: 1.9 $ $Date: 2003/11/06 07:00:53 $
 */

public class Converter {

	// constants
	private String OPTION_FMT = "-fmt";
	private String OPTION_XML = "-xml";
	private String OPTION_OUT = "-out";
	private String OPTION_HSNAME = "-hsname";
	private String OPTION_HSDIR = "-hsdir";
	private String OPTION_PARAM = "-param";
	
	private Properties cmdLineOptions = new Properties();
	private Properties converterParameters = new Properties();

	private void printUsageAndExit() {
		System.out.println("\nUsage: "
			+ "java " + getClass().getName() + " COMMAND OPTIONS [PARAMETERS].. \n"
			+ " COMMAND:\n"
			+ "    -h      : Display this help and exit successfully.\n"
			+ "    -v      : Display version information and exit successfully.\n"
			+ "    -pdf    : Convert input file to pdf.\n"
			+ "    -ps     : Convert input file to ps(postscript).\n"
			+ "    -fo     : Convert input file to fo.\n"
			+ "    -awt    : View pdf output of input file in awt viewer.\n"
			+ "    -html   : Convert input file to output html file.\n"
			+ "    -mhtml  : Convert input file to multiple html files in specified\n"
			+ "              output directory.\n"
			+ "    -mht    : Convert input file to single html file and compile \n"
			+ "              it into a web archive(as MS IE does). \n"
			+ "              The mht format is a MIME 'multipart/related' format\n"
			+ "    -dhtml  : Convert input file to multiple html files with a toc tree\n"
			+ "              in specified output directory.\n"
			+ "    -chm    : Convert input file to multiple html files with the html help\n"
			+ "              content file(.hhc) and html help project file (.hhp).\n"
			+ "              If HTML Help Compiler is installd on the system and the\n"
			+ "              compiler path is specified in aurigadoc.properties a compiled\n"
			+ "              chm file is also generated.\n"
			+ "    -jhelp  : Convert input file to multiple html files with supporting\n"
			+ "              files to make a java help.\n"
			+ "              If indexing option is set to y an index is created.\n"
			+ "              If view options is set to yes then the output is launched in a\n"
			+ "              java help viewer.\n"
			+ "    -ohj    : Convert input file to multiple html files with supporting\n"
			+ "              files required by oracle help for java (OHJ).\n"
			+ "              If indexing option is set to y an index is created.\n"
			+ "              If view options is set to yes then the output is launched in a\n"
			+ "              help viewer.\n"
			+ "    -jhview : View the helpset specified by helpset name and the helpset\n"
			+ "              dir in a java help viewer\n"
			+ "    -man    : Convert input file to unix man source file.\n"
			+ "\n"
			+ " OPTIONS:\n"
			+ "   -XML <file-name>     : The input xml file.\n"
			+ "   -OUT <path>          : The path of output file or directory.\n"
			+ "   -HSNAME helpset-name : The helpset name without the extension.\n"
			+ "                          Required for -jhview option.\n"
			+ "   -HSDIR <path>        : The dir path where the helpset is located.\n"
			+ "                          Required for -jhview option.\n"
			+ " PARAMETERS:\n"
			+ "    -PARAM name=value   : Additional parameter to be passed to the converter.\n\n"
			+ "    The foll parameters are supported\n"
			+ "    " + BaseConverter.XSL + "=<xsl-path>          : "
			+ "the xsl for generating html, pdf,\n"
			+ "                              fo, rtf, awt, etc.\n"
			+ "    " + DHTMLConverter.FRAMESET_XSL + "=<xsl-path> : "
			+ "the xsl for generating the frameset\n"
			+ "                              page in dhtml output.\n"
			+ "    " + DHTMLConverter.TREE_XSL + "=<xsl-path>     : "
			+ "the xsl for generating the toc tree\n"
			+ "                              in dhtml output.\n"
			+ "    " + CHMConverter.HHC_XSL + "=<xsl-path>      : "
			+ "the xsl for generating the html help\n"
			+ "                              content file when converting to chm.\n"
			+ "    " + CHMConverter.HHP_XSL + "=<xsl-path>      : "
			+ "the xsl for generating the html help\n"
			+ "                              project file when converting to chm.\n"
			+ "    " + JHelpConverter.HELPSET_XSL + "=<xsl-path>  : "
			+ "the xsl for generating the help set\n"
			+ "                              file while converting to Java Help.\n"
			+ "    " + JHelpConverter.TOC_XSL + "=<xsl-path>      : "
			+ "the xsl for generating the toc file\n"
			+ "                              while converting to Java Help.\n"
			+ "    " + JHelpConverter.MAP_XSL + "=<xsl-path>      : "
			+ "the xsl for generating the map file\n"
			+ "                              while converting to Java Help.\n"
			+ "    " + JHelpConverter.INDEX_FILES + "=[y|n]       : "
			+ "whether to create an index of the\n"
			+ "                              html files while converting to Java Help.\n"
			+ "    " + JHelpConverter.LAUNCH_VIEWER + "=[y|n]     : "
			+ "whether to launch the output in a\n"
			+ "                              java help viewer while converting to Java Help.\n\n"
			+ " EXAMPLES:\n"
			+ "  html conversion:\n"
			+ "    aurigadoc.sh -html -XML foo.xml -OUT foo.html\n"
			+ "  html conversion using a custom xsl:\n"
			+ "    aurigadoc.sh -html -XML foo.xml -OUT foo.html -PARAM xsl=path-to-xsl\n"
			+ "  java help conversion with indexing and view option:\n"
			+ "    aurigadoc.sh -jhelp -XML foo.xml -OUT foo-jhelp-files -PARAM index_files=y -PARAM launch_viewer=y\n"
			+ "  chm conversion with a custom xsl for generating the html help content file:\n"
			+ "    aurigadoc.sh -chm -XML foo.xml -OUT foo-chm-files -PARAM hhc_xsl=path-to-xsl\n"
			);
		System.exit(-1);
	}

	private void printVersionAndExit() {
		System.out.println(VersionInfo.getInfo());
		System.exit(-1);
	}

	/**
	 * Main method called when invoked from the command line.
	 * @param args command line arguments
	 */
	public static void main(String[] args) {
		try {
			Converter converter = new Converter(args);
		} catch (Exception e) {
			System.out.println("ERROR: " + e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Default constructor.
	 */
	private Converter(String[] args) throws Exception {
		try {
			parseCmdLineOptions(args);
			if (cmdLineOptions.getProperty(OPTION_FMT).equals("-jhview")) {
				launchHelpViewer(cmdLineOptions.getProperty(OPTION_HSNAME), 
					cmdLineOptions.getProperty(OPTION_HSDIR));
			} else {
				convert();
			}
		} catch (IndexOutOfBoundsException ioe) {
			printUsageAndExit();
		} catch (Exception e) {
			throw e;
		}
	}

	private void parseCmdLineOptions(String[] args) 
			throws IndexOutOfBoundsException {
		if (args.length == 0 || args[0].equals("-h")) {
			printUsageAndExit();
		} else if (args[0].equals("-v")) {
			printVersionAndExit();
		} else {
			cmdLineOptions.setProperty(OPTION_FMT, args[0]);
			for (int i = 1; i < args.length; i++) {
				if (i % 2 != 0) {
					if (args[i].equalsIgnoreCase(OPTION_PARAM)) {
						String paramStr = args[i + 1];
						converterParameters.setProperty(
							paramStr.substring(0, paramStr.indexOf("=")),
							paramStr.substring(
								paramStr.indexOf("=") + 1, paramStr.length()
							)	
						);
					} else {
						cmdLineOptions.setProperty(args[i].toLowerCase(), args[i + 1]);
					}
				}
			}
		}
	}

	private int getOutputFormat(String formatStr) {
		int format = 0;
		if (formatStr.equals("-html")) {
			format = Driver.FORMAT_HTML;
		} else if (formatStr.equals("-mhtml")) {
			format = Driver.FORMAT_MHTML;
		} else if (formatStr.equals("-dhtml")) {
			format = Driver.FORMAT_DHTML;
		} else if (formatStr.equals("-jhelp")) {
			format = Driver.FORMAT_JHELP;
		} else if (formatStr.equals("-pdf")) {
			format = Driver.FORMAT_PDF;
		} else if (formatStr.equals("-ps")) {
			format = Driver.FORMAT_PS;
		} else if (formatStr.equals("-rtf")) {
			format = Driver.FORMAT_RTF;
		} else if (formatStr.equals("-awt")) {
			format = Driver.FORMAT_AWT;
		} else if (formatStr.equals("-fo")) {
			format = Driver.FORMAT_FO;
		} else if (formatStr.equals("-chm")) {
			format = Driver.FORMAT_CHM;
		} else if (formatStr.equals("-man")) {
			format = Driver.FORMAT_MAN;
		} else if (formatStr.equals("-mht")) {
			format = Driver.FORMAT_MHT;
		} else if (formatStr.equals("-ohj")) {
			format = Driver.FORMAT_OHJ;
		}
		return format;
	}

	private void convert() throws Exception {
		long t1 = System.currentTimeMillis();
		int format = getOutputFormat(cmdLineOptions.getProperty(OPTION_FMT));
		String inputFile = cmdLineOptions.getProperty(OPTION_XML);
		String outputFile = cmdLineOptions.getProperty(OPTION_OUT);
		DefaultLogger logger = new DefaultLogger();
		
		Driver driver = new Driver();
		driver.setInputFile(cmdLineOptions.getProperty(OPTION_XML));
		driver.setLogger(logger);
		driver.setFormat(format);

		if (format !=  Driver.FORMAT_AWT) {
			driver.setOutputFile(outputFile);
		} 
		driver.setParameters(converterParameters);
		driver.run();
		long t2 = System.currentTimeMillis();
		logger.info("Conversion done in: " + (t2 -  t1) + " msecs.");
	}

	/**
	 * Launches the Java Help Viewer.
	 */
	private void launchHelpViewer(String helpsetName, String helpsetDir) 
			throws Exception {
		new JHelpViewer(helpsetDir, helpsetName);
	}


}
