/* -- $Id: CHMConverter.java,v 1.7 2003/08/29 14:25:41 kshaikh Exp $ --
 Copyright (C) 2001 Auriga Logic <http://www.aurigalogic.com>
 Author: Khurshidali Shaikh <kshaikh@aurigalogic.com>

 ============================================================================
		    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
 ============================================================================

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
package com.aurigalogic.aurigadoc.core;

import java.io.*;

/**
 * This class is used for converting AurigaDoc document to chm.
 * 
 * @author <a href="mailto:kshaikh@aurigalogic.com">Khurshidali Shaikh</a>
 * @version $Revision: 1.7 $ $Date: 2003/08/29 14:25:41 $
 */

public class CHMConverter extends MHTMLConverter {

	/** Identifier for specifying the xsl name which generates the .hhc file */
	public static final String HHC_XSL = "hhc_xsl";

	/** Identifier for specifying the xsl name which generates the .hhp file */
	public static final String HHP_XSL = "hhp_xsl";

	/**
	 * Main method which does the conversion.
	 * @throws Exception in case of any error.
	 */
	public void convert() throws Exception {
		logInfo("Converting to HTML HELP(CHM)..");
		
		// generate the multiple html files
		convertToMHTML(outputFile);
			
		// generate hhp file
		FileOutputStream hhpFos = 
			new FileOutputStream(outputFile + "/adchm.hhp");
		logInfo("Generating HTML Help Project(.hhp) file..");
		doTransform(new FileInputStream(inputFile), 
			hhpFos, getXSLForKey(HHP_XSL, XML2CHMHHP), OUTPUT_HTML);
		hhpFos.close();
			
		// generate hhc file
		FileOutputStream hhcFos = 
			new FileOutputStream(outputFile + "/adchm.hhc");
		logInfo("Generating HTML Help Content(.hhc) file..");
		doTransform(new FileInputStream(inputFile), 
			hhcFos, getXSLForKey(HHC_XSL, XML2CHMHHC), OUTPUT_HTML);
		hhcFos.close();

		// potprocess the hhc file to remove linebreak after li
		// this is not allowed by the chm compiler. 
		logInfo("Post processing the (.hhc) file..");
		String args[] = new String[1];
		args[0] = outputFile + "/adchm.hhc";
		com.aurigalogic.aurigadoc.util.CHMHelper.main(args);

		// if the chm compile is present generate the chm as well
		String chmCompilerPath = getProperty("chm_compiler.path");
		if (chmCompilerPath != null && !chmCompilerPath.equals("")) {
			File chmCompiler = new File(chmCompilerPath);
			if (chmCompiler.exists()) {
				logInfo("Compiling the chm file..");
				// compile exists
				try {
					Runtime rt = Runtime.getRuntime();
					Process p = rt.exec("\"" + chmCompilerPath + "\""
						+ " "  + outputFile + "/adchm.hhp");
					InputStream is = p.getInputStream();
					String line = "";
					BufferedReader br = new 
						BufferedReader(new InputStreamReader(is));
					while ((line = br.readLine()) != null) {
						if (!line.equals("")) {
							System.out.println(line);
						}
					}
				} catch (Exception e) {
					logError("Error compiling chm.", e);
				}
			}
		}
	}

}
