/* -- $Id: PDFConverter.java,v 1.7 2003/09/09 15:51:20 kshaikh Exp $ --
 Copyright (C) 2001 Auriga Logic <http://www.aurigalogic.com>
 Author: Khurshidali Shaikh <kshaikh@aurigalogic.com>

 ============================================================================
		    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
 ============================================================================

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
package com.aurigalogic.aurigadoc.core;

import java.io.*;

import org.apache.fop.apps.Driver;
import org.apache.fop.configuration.Configuration;

import org.xml.sax.InputSource;

import org.apache.avalon.framework.logger.ConsoleLogger;

import java.net.URL;

/**
 * This class is used for converting AurigaDoc document to pdf.
 *
 * @author <a href="mailto:kshaikh@aurigalogic.com">Khurshidali Shaikh</a>
 * @version $Revision: 1.7 $ $Date: 2003/09/09 15:51:20 $
 */

public class PDFConverter extends FOConverter {

	/**
	 * Main method which does the conversion.
	 * @throws Exception in case of error
	 */
	public void convert() throws Exception {
		logInfo("Converting to PDF..");

		FileOutputStream fos = new FileOutputStream(outputFile);
		convertUsingFOP(Driver.RENDER_PDF, 
			getXSLForKey(XSL, XML2PDF), fos);
		fos.close();

	}

	/**
	 * Convert the input file to output stream using the given 
	 * xslFile and the renderer.
	 *
	 * @param renderer the fop renderer to use
	 * @param xslFileUrl the xsl file url
	 * @param outStream the output stream
	 * @throws Exception in case of error
	 */
	protected void convertUsingFOP(int renderer, String xslFileUrl, 
			OutputStream outStream) throws Exception {
		ByteArrayOutputStream foStream = new ByteArrayOutputStream();

		// get the xsl file
		String xslFile = 
			xslFileUrl.substring(xslFileUrl.indexOf("file:/") + 5, 
				xslFileUrl.length());

		// make FO
		createFO(xslFile, new FileInputStream(inputFile), foStream);

		// fopit
		fopIt(renderer, new ByteArrayInputStream(foStream.toByteArray()), 
				outStream);
	}

	private synchronized void fopIt(int renderer, InputStream inStream,
			OutputStream outStream) throws Exception {
		InputSource is = new InputSource(inStream);
		Driver driver = new Driver(new InputSource(inStream), outStream);
		ConsoleLogger cLogger = new ConsoleLogger(ConsoleLogger.LEVEL_INFO);
		driver.setLogger(cLogger);
		driver.setRenderer(renderer);
		// we need to set this otherwise fop is not able to find the images
		// embeded/referenced in the xml if they don't have absolute path
		Configuration.put("baseDir", getBaseURL());
		driver.run();
	}

	/** 
	 * Gets the base url for. The base url is the url of the 
	 * input file directory.
	 */
	private String getBaseURL() {
		try {
			File outFile = new File(inputFile);
			URL baseURL = outFile.getParentFile().toURL();
			return baseURL.toString();
		} catch (Exception e) {
			return "";
		}
	}
}
