<?xml version="1.0" encoding="UTF-8"?>
<!-- $Id: xml2man.xsl,v 1.4 2003/09/16 09:15:15 kshaikh Exp $ -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:output method="text"/>

<!-- B: COMMON VARIABLES -->
<!--
*************************************************************************
Here we declare the document scope variable 
*************************************************************************
-->
<xsl:variable name="dfi" select="/document/document-formatting-info"/>

<xsl:variable name="toc" select="/document/document-body/table-of-content"/>

<xsl:variable name="dmi" select="/document/document-meta-info"/>

<xsl:variable name="nbsp">
<xsl:text> </xsl:text>
</xsl:variable>

<!-- number format for numbering sections -->
<xsl:variable name="snf">
<xsl:choose>
<xsl:when test="$dfi/generate-section-numbers/@number-format">
<xsl:value-of select="$dfi/generate-section-numbers/@number-format"/>
</xsl:when>
<xsl:otherwise>1</xsl:otherwise>
</xsl:choose>
</xsl:variable>

<xsl:variable name="generate-section-numbers">
<xsl:choose>
<xsl:when test="$dfi/generate-section-numbers  = 'no'">no</xsl:when>
<xsl:otherwise>yes</xsl:otherwise>
</xsl:choose>
</xsl:variable>

<!-- E: COMMON VARIABLES -->
<xsl:variable name="slash">\</xsl:variable>
<xsl:variable name="safe-slash" select="concat($slash, '(rs')"/>
<!-- 
*************************************************************************
Template for the document element. This template matches other templates 
from here
*************************************************************************
-->
<xsl:template match="document">.TH <xsl:text> </xsl:text>"<xsl:value-of select="$dmi/title"/>"
	<!-- call the cover page -->
	<xsl:call-template name="cover-page"/>
	<xsl:apply-templates select="document-header"/>	
	<xsl:apply-templates select="document-body"/>
	<xsl:apply-templates select="document-footer"/>	
</xsl:template>
<!-- 
*************************************************************************
Template for the document body. Defines a fo:block and appies other 
templates.
*************************************************************************
-->
<xsl:template match="document-body">
.br
.SH DESCRIPTION
.br
<xsl:apply-templates/>
</xsl:template>
<!-- 
*************************************************************************
This template is handles a section element
*************************************************************************
-->
<xsl:template match="section">
<xsl:variable name="section-name">
<xsl:value-of select="@name"/>
</xsl:variable>
.SH<xsl:text> </xsl:text> 
<!-- navigation -->
<xsl:text> </xsl:text>
<xsl:value-of select="@label"/>
<xsl:text>
</xsl:text>
<xsl:apply-templates/>
</xsl:template>
<!-- 
*************************************************************************
Dummy template to avoid the below template being applied in normal mode
*************************************************************************
-->
<xsl:template match="table-of-content|page-number|link|page-break">
</xsl:template>
<!-- 
*************************************************************************
This template renders the document header
*************************************************************************
-->
<xsl:template match="document-header">
</xsl:template>
<!-- 
*************************************************************************
This template renders the document footer
*************************************************************************
-->
<xsl:template match="document-footer">
</xsl:template>
<!-- 
*************************************************************************
This template renders the cover page
*************************************************************************
-->
<xsl:template name="cover-page">
.SH TITLE
.B<xsl:text> </xsl:text>
<xsl:value-of select="$dmi/title"/>
.SH SYNOPSIS
.B<xsl:text> </xsl:text>
<xsl:value-of select="$dmi/title"/>
<xsl:for-each select="$dmi/attribute">
.br 
.I<xsl:text> </xsl:text>
<xsl:value-of select="@name"/>: <xsl:apply-templates select="."/>
</xsl:for-each>
</xsl:template>
<!-- 
*************************************************************************
This template renders the attribute content
*************************************************************************
-->
<xsl:template match="attribute">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="br">
.br
</xsl:template>

<xsl:template match="b">\fB<xsl:apply-templates/>\fP</xsl:template>

<xsl:template match="i">\fI<xsl:text> </xsl:text>
<xsl:apply-templates/>\fP</xsl:template>

<xsl:template match="ul">
<xsl:text>
</xsl:text>
<xsl:for-each select="li">
<xsl:text>   * </xsl:text>
<xsl:apply-templates/>
<xsl:text>
</xsl:text>
</xsl:for-each>
</xsl:template>

<xsl:template match="ol">
<xsl:text>
</xsl:text>
<xsl:for-each select="li">
<xsl:text>   </xsl:text>
<xsl:value-of select="position()"/>. <xsl:apply-templates/>
<xsl:text>
</xsl:text>
</xsl:for-each>
</xsl:template>

<xsl:template match="li">
<xsl:apply-templates/>
</xsl:template>

<xsl:template match="a">
<xsl:apply-templates/>
</xsl:template>

<xsl:template match="p|div">
.P
<xsl:apply-templates/>
</xsl:template>

<xsl:template match="pre">
.nf
<xsl:apply-templates/>
.fi
</xsl:template>

<xsl:template match="code">
<xsl:apply-templates/>
</xsl:template>

<xsl:template match="text()">
<xsl:variable name="text">
<xsl:call-template name="replace-text">
<xsl:with-param name="str" select="."/>
<xsl:with-param name="replace" select="$slash"/>
<xsl:with-param name="replacement" select="$safe-slash"/>
</xsl:call-template>
</xsl:variable>
<xsl:choose>
<xsl:when test="ancestor::pre">
<xsl:value-of select="$text"/>
</xsl:when>
<xsl:otherwise>
<xsl:value-of select="normalize-space($text)"/>
</xsl:otherwise>
</xsl:choose>
<xsl:text> </xsl:text>
</xsl:template>

<xsl:template match="*|@*|comment()"/>

<!-- B: STRING FUNCTIONS -->
<!-- common space characters -->
<xsl:variable name="lf">
<xsl:text>
</xsl:text>
</xsl:variable>
<xsl:variable name="tab">
<xsl:text>	</xsl:text>
</xsl:variable>
<xsl:variable name="lftab">
<xsl:copy-of select="$lf"/>
<xsl:copy-of select="$tab"/>
</xsl:variable>
<!-- template to replace characters like tab, newline, etc -->
<xsl:template name="make-safe">
<xsl:param name="str"/>
<xsl:variable name="replacement" select="concat('','')"/>
<xsl:variable name="tab-safe">
<xsl:call-template name="replace-text">
<xsl:with-param name="str" select="$str"/>
<xsl:with-param name="replace" select="$tab"/>
<xsl:with-param name="replacement" select="$replacement"/>
</xsl:call-template>
</xsl:variable>
<xsl:variable name="lf-safe">
<xsl:call-template name="replace-text">
<xsl:with-param name="str" select="$tab-safe"/>
<xsl:with-param name="replace" select="$lf"/>
<xsl:with-param name="replacement" select="$replacement"/>
</xsl:call-template>
</xsl:variable>
<xsl:variable name="safe">
<xsl:call-template name="replace-text">
<xsl:with-param name="str" select="$lf-safe"/>
<xsl:with-param name="replace" select="$lftab"/>
<xsl:with-param name="replacement" select="$replacement"/>
</xsl:call-template>
</xsl:variable>
<xsl:value-of select="$safe"/>
</xsl:template>
<!-- template to replace text -->
<xsl:template name="replace-text">
<xsl:param name="str"/>
<xsl:param name="replace"/>
<xsl:param name="replacement"/>
<xsl:choose>
<xsl:when test="contains($str, $replace)">
<xsl:value-of select="concat(substring-before($str, $replace), $replacement)"/>
<xsl:call-template name="replace-text">
<xsl:with-param name="str" select="substring-after($str, $replace)"/>
<xsl:with-param name="replace" select="$replace"/>
<xsl:with-param name="replacement" select="$replacement"/>
</xsl:call-template>
</xsl:when>
<xsl:otherwise>
<xsl:value-of select="$str"/>
</xsl:otherwise>
</xsl:choose>
</xsl:template>
<!-- E: STRING FUNCTIONS -->
</xsl:stylesheet>
